/**************************************************************************
 shoot_growth_criteria.c

  This file is to calculate the differences between the averages of  
  measured and simulated shoot growth for hours 3-6, 9-12, 13-16, and 21-24
  on days 179-181 or 182-184. 

**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "UserSettings.h"
#include "data.h"



int main(void)
{
   int i, j, k;     /* generic counters */

   double x[NUMPARAMS];              /* parameters */
   double sim_data[PERIOD][24];      /* hourly shoot growth */
   double dayinc;                    /* daily increment on the previous day */
   double p;                         /* for parameter values */
   double dfct=0.0, prev_dfct=0.0;   /* for water deficit */
   double uptake;                    /* water uptake */

   FILE *param_fptr;   /* to read in from PARAM */
   FILE *crit_fptr;    /* to write out to CRITERIA */



   /* Open the parameter file. */

   if ( ( param_fptr = fopen(MODEL_INPUT_CALL, "r") ) == NULL )
   {
      printf("Cannot open the parameter file.\n");
      exit(1);
   }

   /* Open the criteria file. */

   if ( ( crit_fptr = fopen(CRIT_OUT_CALL,"w") ) == NULL )
   {
      printf("Cannot open the criteria file.\n");
      exit(1);
   }


   /* Throw out the item names in the parameter file. */

   for (i=0; i<NUMPARAMS; i++)   fscanf(param_fptr, "%*s");


   /* Print the parameter names at the top of the criteria file. */

   for (i=1; i<=NUMPARAMS; i++)   fprintf(crit_fptr, "%s%d  ", "x", i);


   /* Print the criteria names at the top of the criteria file. */

   for (i=1; i<=NUMCRITERIA; i++)   fprintf(crit_fptr, "%s%d  ", "d", i);
   fprintf(crit_fptr, "\n");



   /* Read in the parameters from the input file, and write them out to the output file. 
Then, calculate the differences between measured and simulated data. */

   for (i=0; i<POP_SIZE; i++)
   {
      dfct = 0.0;
      prev_dfct = 0.0;


   /* Read in the parameters from the input file. */

      for (j=0; j<NUMPARAMS; j++)
      {
         fscanf(param_fptr, "%lf", &p);
         fprintf(crit_fptr, "%lf ", p);
         x[j] = p;
      }


   /* Calculate the simulation data and the differences from the measured ones at the time 
when they take the daily minimums and maximums. */

      for (j=0; j<PERIOD; j++)
      {

   /* Calculate daily increment on the previous day. */

         dayinc = 0.0;

         if (j == 0)
         {
            for (k=0; k<24; k++)   dayinc += growth[FIRST_DAY-1][k];
         }

         else
         {
            for (k=0; k<24; k++)   dayinc += sim_data[j-1][k];
         }


   /* Simulate shoot growth. */

         for (k=0; k<24; k++)
         {
            if (dfct < 0)   dfct = 0.0;

            sim_data[j][k] = average_temp[j+FIRST_DAY+1][k] * x[0] 
                             - average_temp[j+FIRST_DAY][k] * x[1] 
                             + ( total_radiat[j+FIRST_DAY+1][k] * x[2] 
                             + total_radiat[j+FIRST_DAY][k] * x[3] ) / 360
                             + x[4] * (prev_dfct-dfct) * dayinc;

            uptake = dfct * x[5];
            prev_dfct = dfct;
            dfct = prev_dfct + transp[j+FIRST_DAY][k] - uptake;

         }   /* end for k loop */
	  
      }   /* end for j loop */


   /* Write out the difference between the three hour averages of measured and simulated data
as criteria. */

      for (j=ASSESSMENT_START_DAY; j<=ASSESSMENT_LAST_DAY; j++)
      {  
         fprintf(crit_fptr, "%lf ", 
                 ( (sim_data[j][2] + sim_data[j][3] + sim_data[j][4] + sim_data[j][5]) 
                    - (growth[j+FIRST_DAY][2] + growth[j+FIRST_DAY][3] 
                       + growth[j+FIRST_DAY][4] + growth[j+FIRST_DAY][5]) ) / 4);
         fprintf(crit_fptr, "%lf ", 
                 ( (sim_data[j][8] + sim_data[j][9] + sim_data[j][10] + sim_data[j][11])
                    - (growth[j+FIRST_DAY][8] + growth[j+FIRST_DAY][9] 
                       + growth[j+FIRST_DAY][10] + growth[j+FIRST_DAY][11]) ) / 4);
         fprintf(crit_fptr, "%lf ", 
                 ( (sim_data[j][12] + sim_data[j][13] + sim_data[j][14] + sim_data[j][15])
                    - (growth[j+FIRST_DAY][12] + growth[j+FIRST_DAY][13] 
                       + growth[j+FIRST_DAY][14] + growth[j+FIRST_DAY][15]) ) / 4);
         fprintf(crit_fptr, "%lf ", 
                 ( (sim_data[j][20] + sim_data[j][21] + sim_data[j][22] + sim_data[j][23])
                    - (growth[j+FIRST_DAY][20] + growth[j+FIRST_DAY][21] 
                       + growth[j+FIRST_DAY][22] + growth[j+FIRST_DAY][23]) ) / 4);
      }

      fprintf(crit_fptr, "\n");

   }   /* end for i loop */


   /* Close opened files */

   fclose(crit_fptr);
   fclose(param_fptr);

}   /* end of main */