/*******************************************************************
UserArrays.h
One of two header files that the user must edit before compiling
the ParetoEvolve code (UserSettings.h, UserArrays.h).  
In this file the user defines the arrays that hold the model's input and output,
the feasible search space, and the discrepancy measures.

This file is 'included' in the 'PARETO_misc.c' file of program functions.
********************************************************************        
    Editing History: 
    26 Feb 2006 - created to simplify code use;
            Joel H. Reynolds, joel_reynolds@fws.gov
********************************************************************
NOTES:
IMPORTANT: THE USER MUST ADD OR SUBTRACT ELEMENTS TO THE ARRAYS AS NEEDED.
This separate header file is used to avoid declaring these arrays multiple
times.  Someone familiar with precompiler commands could surely create
a cleaner solution.
*********************************************************************/


#ifndef _UserArrays_h
#define _UserArrays_h 1

/****** APPLICATION-SPECIFIC ARRAY DECLARATIONS ******/
    /***** NAMES ******/
    /* PARAMETER NAMES to be used in printing out search results */
   /* NAMES are LIMITED to 10 characters!! ADD ELEMENTS AS NEEDED.*/
char pnames[NUMPARAMS][11] = {"x1","x2","x3","x4","x5","x6"};

   /* CRITERIA NAMES to be used in printing out search results */
   /* NAMES are LIMITED to 10 characters!! ADD ELEMENTS AS NEEDED.*/
char cnames[NUMCRITERIA][11] = {
        "d1", "d2", "d3", "d4", "d5", "d6", "d7", "d8", "d9", "d10",
        "d11", "d12"};
      
      /***** SEARCH RANGES AND SPECIFICS ******/
      /* This array should have a 3 component row-vector for each parameter. 
         The first element is the MINIMUM value of that parameter's search range, 
         the second component is the MAXIMUM value of the search range, and 
         the third component is the MINIMUM STEPSIZE for the parameter. 
         E.g., '{0.0, 0.4, 0.001}' implies feasible search values of 
         0.0, 0.001, 0.002, 0.003, ..., 0.399, 0.400; (400 possible search
         values for this parameter).
         The first row is associated with the first parameter in pnames[], 
         the second row with the second parameter, etc. 
      ADD ELEMENTS AS NEEDED.*/
double psearch[NUMPARAMS][3] = {
      /* {Min, Max, Min Step}, */  
      {0.0, 0.4, 0.001},   /* search for param x1 */
      {0.0, 0.1, 0.001},   /* search for param x2 */
      {0.0, 0.1, 0.001},   /* search for param x3 */
      {0.0, 0.1, 0.001},   /* search for param x4 */
      {0.0, 1.0, 0.001},   /* search for param x5 */
      {0.0, 1.0, 0.001}    /* search for param x6 */
   };
   
   /***** DISCREPANCY MEASURE SPECIFICS ******/
   /* NOTE: CURRENTLY CODE ASSUMES that criteria error functions are either 
      ALL binary or ALL continous.*/
   /* This array should have a 2 component row-vector for each criterion. 
      If BINARY discrepancy measures, then each element is the Acceptable 
      range for each criteria: [Min, Max].  E.g, {5, 32}.
      If CONTINUOUS discrepancy measures, then the first component of 
      each element is the Target Value and the second component is ignored:
      {Target, trash}. E.g., {18,0}. 
      The first row is associated with the first criterion in cnames, 
      the second row with the second criterion, etc.*/ 
   /* ADD ELEMENTS AS NEEDED.*/
double assessinfo[NUMCRITERIA][2] = {
   /* BINARY? {Min, Max}. CONTINUOUS? {Target, trash} */
      {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, 
      {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, {-0.41,0.41}, 
      {-0.41,0.41}, {-0.41,0.41}
   };
/******************************************************************/

#endif 



