/********************************************************************
Pareto_misc.c includes all the files that need to access UserArrays.h
                                     6 Oct 08 JHR remove debug flags
********************************************************************/

#include "ParetoEvolve.h"
#include "UserSettings.h"
#include "UserArrays.h"



/*******************functions from PARETO_main.c **********************/


/***************************** Read In Pareto *************************/
/* This function is called from main() to restart the search if it has 
    been stopped before STOP_GEN. The restart must occur after generation 0. 
    The function opens the file paretofilename and reads in the last 
    pareto frontier, dynamically allocating memory as required.       */
/**********************************************************************/

void readin_pareto_frontier(char paretofilename[15], 
    struct pset **ptr_to_Frontier_first_location)
{
    FILE *paretofileptr;            /* pointer to pareto.set file */
    struct pset *pareto_member;     /* pointer to Historical Pareto Frontier's member */


    /* Open the Historical Pareto Frontier from the last generation before 
    the restart. */

    paretofileptr=fopen(paretofilename,"r");

    /* Send an error message and stop search if there is a problem opening the 
    pareto.set file. */

    if (paretofileptr==NULL)
    { 
        perror(paretofilename);
        exit(EXIT_FAILURE);
    }
 
    /* Allocate memory to read in the first Frontier member. */

    if (*ptr_to_Frontier_first_location==NULL) 
    { 
        *ptr_to_Frontier_first_location=(struct pset *)malloc(sizeof(struct pset));
        pareto_member = *ptr_to_Frontier_first_location;
        pareto_member->next=NULL;
        pareto_member->psim=NULL;
    }


    /* Throw out the file name listing. */
 
        fscanf(paretofileptr,"%*s%*s");
 

    /* Read in the assessment vector of the first member of the 
    Historical Pareto Frontier. */
 
        readin_assessment_vector(paretofileptr, pareto_member,1);


    /* Read in the simulation information of the first member of the 
    Historical Pareto Frontier. */

        readin_psims(paretofileptr, pareto_member);
  

    /* Read the next item from file. This item is referred to as a token. 
    If the token is the end of file, get an error. Else read in 'Assessment' 
    part of header label and continue reading in Pareto Frontier members.
    Loop through all Frontier members until the end of file. */
 
 
    while (AnotherMemberTest(paretofileptr)==1) 
    { 

   /* If the next token is 'Assessment', then there is another Historical 
   Pareto Frontier member. Allocate memory, then read in the rest of the 
   next member's assessment vector. */
   
        pareto_member->next=(struct pset *) malloc(sizeof(struct pset));
        pareto_member=pareto_member->next;
        pareto_member->next = NULL;
        pareto_member->psim=NULL;
   
        readin_assessment_vector(paretofileptr,pareto_member,0);

    /* Read in the simulation information of the next member of the 
    Historical Pareto Frontier. */

        readin_psims(paretofileptr,pareto_member);

    } /* End of reading additional Historical Pareto Frontier members while loop */

    /* Close the Historical Pareto Frontier file. Return to main function. */

    fclose(paretofileptr);

} /* End of readin_pareto_frontier. */




/****************** readin_assessment_vector ***********************/
/* This function reads in the Historical Pareto Frontier's member's
    assessment vector. The Full_header flag tells how much header is 
    left to toss out before getting to assessment vector in the 
    Historical Pareto Frontier file.*/
/*******************************************************************/

void readin_assessment_vector(FILE *paretofileptr, 
        struct pset *pareto_member, int Full_header)
{
    int i;

    /* Throw out the header in the file. */

    if (Full_header==1) 
    { 
        fscanf(paretofileptr,"%*s%*s%*s%*s");
    }
    else 
    {
        fscanf(paretofileptr,"%*s%*s");
    }
   

    /* Throw out the criteria labels in the file.  */
   
    for (i=0; i<NUMCRITERIA; i++)
    {
        fscanf(paretofileptr,"%*s");
    }
 

    /* Read in the Frontier member's assessment vector. */
   
    for (i=0; i<NUMCRITERIA; i++)
    {
        fscanf(paretofileptr,"%lf", &pareto_member->assess.crit[i]);
    }
 
    /* Return to calling function, readin_pareto_frontier.  */

} /* End of readin_assessment_vector. */





/*********************** read in psims **********************/
/* This function reads in simulation information. If there are 
    no errors and  '-999999.999' has not been encountered, then 
    allocate memory and record information. Repeat loop until 
    the read hits the record divider '-999999.999'.         */
/************************************************************/

void readin_psims(FILE *paretofileptr, struct pset *pareto_member)
{           
   struct psiminfo  *current_psim; /* pointer to the current simulation 
                                    used to move down the psim list. */ 
   int i, erc=0;
   double testtoken;


    /* Throw out each simulation's section divider.  */

        fscanf(paretofileptr,"%*s%*s%*s");
 
    /* Throw out the parameterization and criteria names.  */

        for (i=0; i< (NUMPARAMS + NUMCRITERIA); i++)
        {
            fscanf(paretofileptr,"%*s");
        }

     /*There is at least one simulation, so allocate memory for recording it. */

        pareto_member->psim = (struct psiminfo *) malloc (sizeof(struct psiminfo));
        (pareto_member->psim)->next=NULL;

    /* Set the current_psim to the beginning of the psim list */

        current_psim=pareto_member->psim;

    /* Read in the parameterization for the first simulation. */
  
        for(i=0; i<NUMPARAMS; i++) 
        { 
            if (fscanf(paretofileptr,"%lf", 
                 &(current_psim)->paramvalues.par[i]) !=1) erc=1;
        }

    /* Read in the criteria results for the first simulation. */
  
        for(i=0; i<NUMCRITERIA; i++) 
        { 
            if (fscanf(paretofileptr,"%lf", 
                 &(current_psim)->cresults.crit[i]) !=1) erc=1;
        }

    /* Finished with first simulation. Now to check for more.
     Read in next token - if it is '-999999.999' then the end of psim info 
     has been reached, so return to calling function, readin_pareto_frontier.
     Else, allocate memory for and store next simulation information. */
  

        fscanf(paretofileptr,"%lf",&testtoken);
  
    /* If the record divider has not been encountered, read in the next 
        simulation. */

        while (!(testtoken==-999999.999))
        { 
    /* Allocate memory for the next simulation. */

        current_psim->next = (struct psiminfo *) malloc(sizeof(struct psiminfo));
        current_psim = current_psim->next;
        current_psim->next = NULL;

    /* Record the first entry, currently stored in the testtoken. */

        current_psim->paramvalues.par[0]=testtoken;

    /* Read in the rest of the parameterization  of the current simulation.  */

        for(i=1; i<NUMPARAMS; i++) 
            { 
            if (fscanf(paretofileptr,"%lf", 
                 &(current_psim)->paramvalues.par[i]) !=1) erc=1;
            }

    /* Read in the criteria results of the current simulation. */

        for(i=0; i<NUMCRITERIA; i++) 
            { 
            if (fscanf(paretofileptr,"%lf", 
                 &(current_psim)->cresults.crit[i]) !=1) erc=1;
            }

    /* Send a message if the Frontier file records are incomplete. */

        if (erc) printf("Error reading input file - incomplete records.");

    /* Read in the first string of the next section of the file as the 
    prelude to the while loop test. */

        fscanf(paretofileptr,"%lf",&testtoken);

        } /* end of while loop - failed test, no more psims for this list */
 
}       /* End of readin_psims. */





/******************* FUNCTION: AnotherMemberTest *********************/
/* This function reads in the next string and checks to see if is the 
    word 'Assessment'. It is calling by readin_pareto_frontier.
        If Yes - return '1' to read in next frontier member.
        If No - return '0' to stop reading from file.                */
/*********************************************************************/

int AnotherMemberTest(FILE *paretofileptr)
{
    char teststring[11];
    char target[]="Assessment";


    fscanf(paretofileptr,"%s",teststring);
    if (strcmp(teststring,target)==0)
      return(1);
    else return (0);
 
}





/**************** FUNCTION: initial_input ************************/
/* This function creates an initial population of parameterizations 
for the first generation of the evolutionary computation search. 
It is called by main.                                            */
/*****************************************************************/

void initial_input(void)
{

    FILE *op;
    int i,j;
    double scale=0.0000;


    /* Open up a file in which to write the parameterizations. */

    if( (op=fopen(MODEL_INPUT_CALL,"w") ) == NULL)
        printf("can not open input.txt file.\n");


    /* Print the parameter names at the top of the file.  */

    for(j = 0; j< NUMPARAMS; j++)
      {
       fprintf(op,"  %s",pnames[j]);
      }
    fprintf(op,"\n");

    /* Loop through the parameterization selection for each simulation 
    (individual) in the population */

    for(i=0; i < POP_SIZE; i++)
       {

    /* Randomly select the parameter estimates for each parameter of the model. The
        vector of parameter estimates is referred to as a parameterization. The
        parameter estimates are selected within each parameter's search range. */

        for(j = 0; j< NUMPARAMS; j++)
        {
         scale=psearch[j][2];
         fprintf(op," %f",(psearch[j][0]+ 
             rinteger(rand01()*(psearch[j][1] - psearch[j][0])*(1/scale) )*scale) );

        }  /* End of parameterization loop. */
    /* adds a new line between parameterizations. */
        fprintf(op,"\n");

      }   /* End of population loop. */

    /* Close the file and return to main. */

    fclose(op);

}   /* End of initial_input.   */


/****************** functions from PARETO_update.c ******************/

        
/************************ criteria_assess *************************/
/* This function creates the assessment vector of the latest simulation 
results. The constant BINARY declares whether continuos or binary error 
measures are used. The array assessinfo contains either the acceptance 
range (BINARY ==1) or target values(BINARY == 0).

NOTE: the nondominated ranking algorithm used in the selection process 
(Srinivas & Deb 1995) is only based on the rank of the assessment vector
components across the population, thus there is no need to decide on any 
error (distance/cost) measure other than the absolute value (L1 norm).
The acceptance range/target values are defined by the user in UserArrays.h. */
/***************************************************************************/

void criteria_assess(struct siminfo *latest_sim_ptr)
{
    int i;
 
    /* For binary error measures, the assessment vector records whether
    the criteria vector elements fall within the criteria target ranges. */

    if(BINARY)
     {
      for(i=0; i < NUMCRITERIA; i++)
        {
            if ( (latest_sim_ptr->cresults.crit[i] >= assessinfo[i][0]) 
                && (latest_sim_ptr->cresults.crit[i] <= assessinfo[i][1]) )
                {
                    latest_sim_ptr->assess_res.crit[i] = 1.00;
                } 
            else
                {
                    latest_sim_ptr->assess_res.crit[i] = 0.00;
                }   
        }
      }

    /* For continuous error measures, the assessment vector is the Euclidean
    distance between the criteria vector and the criteria's vector of target values. */
    
    else
     {
        for(i=0; i < NUMCRITERIA; i++)
            {
            latest_sim_ptr->assess_res.crit[i] = 
                fabs(latest_sim_ptr->cresults.crit[i] - assessinfo[i][0]);
            }
     }

    /* Return to calling function, evaluation.  */

}   /* End of criteria_assess.  */




/********************** FUNCTION: print_pareto **************************/
/* This function writes out the assessment vectors, parameterizations, 
    and the criteria results vectors to the Historical Pareto Frontier. */
/************************************************************************/

void print_pareto(struct pset *Frontier_first_ptr, int gen)
{
    FILE *op;
    struct pset *pareto_ptr;
    struct psiminfo *temp;
    extern char paretofile[100];
    int i;

    /* Open file in which to write the Historical Pareto Frontier.  */

    sprintf(paretofile,"%s%d",FRONTIER_CALL,gen);
    op=fopen(paretofile,"w");
    fprintf(op,"Pareto Frontier:\n");

    /* Start the pointer at the top of the Frontier list. */
    
    pareto_ptr = Frontier_first_ptr;

    /* Run down the list of Frontier members. Each member represents a 
    different assessment vector. The -999999.999 number is used to discern 
    the different members in the file if the search is restarted.*/

    while(pareto_ptr!=NULL) 
    {

    /* Put a header at the top of the file. */

        fprintf(op,"-999999.999 Assessment Vector 999999.999 \n");

    /* Print the criteria labels into the file. */

        for(i=0; i < NUMCRITERIA; i++)
          {
            fprintf(op,"   %s",cnames[i]);
          }
        fprintf(op,"\n");

    /* Print the assessment vector for the Frontier member into the file. */


        for(i=0; i < NUMCRITERIA; i++)
          {
            fprintf(op,"   %4.2f",pareto_ptr->assess.crit[i]);
          }
        fprintf(op,"\n");

    /* Print the labels for the simulation information per Frontier member 
        into the file. */

        fprintf(op,"=============== Simulations =============== \n");
        for(i=0; i < NUMPARAMS; i++)
          {
            fprintf(op,"  %s",pnames[i]);   
          }

        for(i=0; i < NUMCRITERIA; i++)
          {
            fprintf(op,"  %s  ",cnames[i]);
          }

        fprintf(op,"\n");

    /* Loop through all the simulations that have the same assessment vector. */

        temp=pareto_ptr->psim;

        while(temp!=NULL)
        {

    /* Print the parameterization per simulation into the file.  */

         for(i=0; i < NUMPARAMS; i++)
           {
            fprintf(op," %.3f",temp->paramvalues.par[i]);
                        
           }

    /* Print the criteria results per simulation into the file.  */

         for(i=0; i < NUMCRITERIA; i++)
           {
            fprintf(op," %.5f",temp->cresults.crit[i]);
           }
         fprintf(op,"\n");

    /* Increment to next simulation in list for this Frontier member. */

         temp=temp->next;

        }  /* End of temp while loop, simulations list. */


    /* Increment to next Frontier member. */

        pareto_ptr=pareto_ptr->next;

    }  /* End of while pareto list loop. */

    fprintf(op,"-999999.999 ========== ========== ========== \n");

    /* Close the file and return to calling function, main. */

    fclose(op);

}   /* End of print_pareto.  */




/*********** functions from PARETO_breed.c ******************/

/******************* single_mutate **************************/ 
/* This function takes a current parameter estimate (one element 
    in a parameterization) and mutates it into a new parameter 
    estimate. The new parameter estimate must fall within the 
    paramter's search range.                                */
/************************************************************/

void single_mutate(double *param_ptr, int j, int gen)
{

    double paramtemp;


    paramtemp = *param_ptr;

    /* Randomly select to increase or decrease the the current parameter
    estimate. If the random selection is less than or equal to 0.5, increase 
    the current parameter estimate. */

    if (rand01() <= 0.5)    
      {

    /* Mutate by increasing the current parameter estimate if it is 
    less than the upper boundary of the parameter's search range.  */

       if((psearch[j][1] - paramtemp) > 0.0)    paramtemp += mutation(gen,j,0,param_ptr);

    /* If the parameter estimate already equals the upper bound, mutate by 
       decreasing the current parameter estimate.  */ 

       else     paramtemp -= mutation(gen,j,1,param_ptr);

     }

    /*  */
    else 
      {

    /* Else if the random selection is greater than 0.5, decrease the current 
    parameter estimate. Mutate by decreasing the current parameter estimate if it is 
    greater than the lower boundary of the parameter's search range.  */

       if((paramtemp - psearch[j][0]) > 0.0)    paramtemp -= mutation(gen,j,1,param_ptr);

    /* If the parameter estimate already equals the lower bound, mutate by 
       increasing the current parameter estimate.  */

       else   paramtemp += mutation(gen,j,0,param_ptr);
     }

    /* Return the new parameter estimate to th calling function, 
    mutator or new_individual_check. */

        *param_ptr = paramtemp;
        
}   /* End of single_mutate.  */




/*********************** mutation ******************************/
/* This function calculates the mutation to be added or subtracted
    from the current paramter estimate to create a new parameter
    estimate.
   The evolutionary computation search is on a Cartesian grid system, 
    so the scale is the increment size between each point on the grid. 
    A mutation is a fraction of the possible range between the current
    paramter estimate and a boundary of the paramter's search range. */
/***************************************************************/

double mutation(int gen,int j, int rtype, double *param_ptr)
{
    double range,scale;
    double change;
    double temp1,temp2;

    /* The rtype indicates which part of the parameter's search range 
    to select (to mutate to) the new parameter estimate.    */


    /* If rtype=0, mutate to part of search range above the current 
    parameter estimate. */

        if(rtype==0)
        {
         range = psearch[j][1] - *param_ptr;
        }

    /* Else, mutate to part of search range above the current parameter estimate.  */

        else
        {
          range = *param_ptr - psearch[j][0];
        }

        scale = psearch[j][2];

    /* The mutations get smaller as the number of generations approaches NUM_GEN.  */

        temp1= pow((1-gen/GEN_NUM),BETA);
        temp2 =rand01();
        change = range*(1-pow(temp2,temp1));

    /* The mutation must be divided by the scale, rounded to an integer, then 
    multiplied by the scale, to select a new parameter estimate that occurs on 
    the grid. */

        change=rinteger(change*(1/scale));
        change=change*scale;

    /* Make sure the mutation is nonzero. */

        if(change == 0.0) change = scale;

    /* Return mutation to calling function, single_mutate.  */

        return(change);

}   /* End of mutation. */



/************************ offspring_output ********************/
/*  This function writes the new population of parameterizations 
produced by the breeding functions, the next generation to be 
run in the model, to the model input file.                    */
/**************************************************************/

void offspring_output(struct param parents[POP_SIZE])
{
    FILE *op;
    int i,j;


    /* Open the file in which to write the new parameterizations to 
    be used as the next generation. */

        op=fopen(MODEL_INPUT_CALL,"w");

    /* Print the parameter names at the top of the file. */

        for(i=0;i < NUMPARAMS; i++)
        {
            fprintf(op,"  %s",pnames[i]);
        }
        fprintf(op,"\n");


    /* Loop through the population and print the parameterizations 
        into the file. */

        for(i=0; i < POP_SIZE; i++)
        {
            for(j=0; j < NUMPARAMS; j++)
            {
             fprintf(op,"  %lf",parents[i].par[j]);   
            }
            /* add a new line between parameter sets. */
            fprintf(op,"\n"); 
      }

    /* Close the file and return to the calling function, breed.  */
        
        fclose(op);

}   /* End of offspring_output.  */

/************************ FUNCTION: rinteger *****************/
/* This functin rounds a real number to an integer.          */
/*************************************************************/

int rinteger(double xvalue)
{
    if ( (xvalue - floor(xvalue)) < 0.5)    return (int) floor(xvalue);
    else                                    return (int) ceil(xvalue);
}


